@@ -1,6 +1,7 @@
 /*! Copyright [Amazon.com](http://amazon.com/), Inc. or its affiliates. All Rights Reserved.
 SPDX-License-Identifier: Apache-2.0 */
 import * as fs from "fs";
+import * as util from "util";
 import SwaggerParser from "@apidevtools/swagger-parser";
 import { parse } from "ts-command-line-args";
 import * as ejs from "ejs";
@@ -16,6 +17,7 @@ import _orderBy from "lodash/orderBy";
 import _uniq from "lodash/uniq";
 import _uniqBy from "lodash/uniqBy";
 import _isEqual from "lodash/isEqual";
+import _cloneDeepWith from "lodash/cloneDeepWith"
 import { OpenAPIV3 } from "openapi-types";
 import * as parseOpenapi from "parse-openapi";
 import { getOperationResponses } from "parse-openapi/dist/parser/getOperationResponses";
@@ -301,6 +303,7 @@ const splitAndWriteFiles = (renderedFileContents: string[], outputPath: string)
 
 // Model types which indicate it is composed (ie inherits/mixin's another schema)
 const COMPOSED_SCHEMA_TYPES = new Set(["one-of", "any-of", "all-of"]);
+const COLLECTION_TYPES = new Set(["array", "dictionary"]);
 const PRIMITIVE_TYPES = new Set(["string", "integer", "number", "boolean", "null", "any", "binary", "void"]);
 
 /**
@@ -359,6 +362,10 @@ const toTypeScriptType = (property: parseOpenapi.Model): string => {
       return `Array<${property.link && property.link.export !== "enum" ? toTypeScriptType(property.link) : property.type}>`;
     case "dictionary":
       return `{ [key: string]: ${property.link && property.link.export !== "enum" ? toTypeScriptType(property.link) : property.type}; }`;
+    case "one-of":
+    case "any-of":
+    case "all-of":
+      return property.name;
     default:
       return property.type;
   }
@@ -412,6 +419,10 @@ const toJavaType = (property: parseOpenapi.Model): string => {
       return `${property.uniqueItems ? 'Set' : 'List'}<${property.link && property.link.export !== "enum" ? toJavaType(property.link) : property.type}>`;
     case "dictionary":
       return `Map<String, ${property.link && property.link.export !== "enum" ? toJavaType(property.link) : property.type}>`;
+    case "one-of":
+    case "any-of":
+    case "all-of":
+      return property.name;
     default:
       // "any" has export = interface
       if (PRIMITIVE_TYPES.has(property.type)) {
@@ -461,6 +472,10 @@ const toPythonType = (property: parseOpenapi.Model): string => {
       return `List[${property.link && property.link.export !== "enum" ? toPythonType(property.link) : property.type}]`;
     case "dictionary":
       return `Dict[str, ${property.link && property.link.export !== "enum" ? toPythonType(property.link) : property.type}]`;
+    case "one-of":
+    case "any-of":
+    case "all-of":
+      return property.name;
     default:
       // "any" has export = interface
       if (PRIMITIVE_TYPES.has(property.type)) {
@@ -483,7 +498,7 @@ const mutateModelWithAdditionalTypes = (model: parseOpenapi.Model) => {
   (model as any).javaType = toJavaType(model);
   (model as any).pythonName = toPythonName('property', model.name);
   (model as any).pythonType = toPythonType(model);
-  (model as any).isPrimitive = PRIMITIVE_TYPES.has(model.type);
+  (model as any).isPrimitive = PRIMITIVE_TYPES.has(model.type) && !COMPOSED_SCHEMA_TYPES.has(model.export) && !COLLECTION_TYPES.has(model.export);
 };
 
 interface MockDataContext {
@@ -602,6 +617,8 @@ const _ensureModelLinks = (spec: OpenAPIV3.Document, modelsByName: {[name: strin
       if (modelsByName[name] && !model.link) {
         model.link = modelsByName[name];
       }
+    } else if (model.link && typeof schema.additionalProperties !== 'boolean') {
+      _ensureModelLinks(spec, modelsByName, model.link, schema.additionalProperties, visited);
     }
   } else if (model.export === "array" && 'items' in schema && schema.items) {
     if (isRef(schema.items)) {
@@ -609,6 +626,8 @@ const _ensureModelLinks = (spec: OpenAPIV3.Document, modelsByName: {[name: strin
       if (modelsByName[name] && !model.link) {
         model.link = modelsByName[name];
       }
+    } else if (model.link) {
+      _ensureModelLinks(spec, modelsByName, model.link, schema.items, visited);
     }
   }
 
@@ -649,7 +668,7 @@ const filterInlineCompositeSchemas = (schemas: (OpenAPIV3.SchemaObject | OpenAPI
   let inlineSchemaIndex = 0;
   return schemas.flatMap((s, i) => {
     if (hasSubSchemasToVisit(s)) {
-      const subSchema: SubSchema = { nameParts: [...nameParts, `${namePartPrefix}${inlineSchemaIndex === 0 ? '' : inlineSchemaIndex}`], schema: s, prop: `${prop}.[${i}]` };
+      const subSchema: SubSchema = { nameParts: s.title ? [_upperFirst(_camelCase(s.title))] : [...nameParts, `${namePartPrefix}${inlineSchemaIndex === 0 ? '' : inlineSchemaIndex}`], schema: s, prop: `${prop}.[${i}]` };
       inlineSchemaIndex++;
       return [subSchema];
     }
@@ -700,13 +719,12 @@ const buildData = async (inSpec: OpenAPIV3.Document, metadata: any) => {
   // In order for the new generator not to be breaking, we apply the same logic here, however this can be removed
   // in future since we have control to avoid the duplicate handlers while allowing an operation to be part of
   // multiple "services".
-  let spec = JSON.parse(JSON.stringify(inSpec, (key, value) => {
+  let spec = _cloneDeepWith(inSpec, (value, key) => {
     // Keep only the first tag where we find a tag
     if (key === "tags" && value && value.length > 0 && typeof value[0] === "string") {
       return [value[0]];
     }
-    return value;
-  })) as OpenAPIV3.Document;
+  }) as OpenAPIV3.Document;
 
   // Ensure spec has schemas set
   if (!spec?.components?.schemas) {
@@ -718,21 +736,34 @@ const buildData = async (inSpec: OpenAPIV3.Document, metadata: any) => {
     };
   }
 
-  // "Hoist" inline response schemas
+  // "Hoist" inline request and response schemas
   Object.entries(spec.paths ?? {}).forEach(([path, pathOps]) => Object.entries(pathOps ?? {}).forEach(([method, op]) => {
     const operation = resolveIfRef(spec, op);
-    if (operation && typeof operation === "object" && "responses" in operation) {
-      Object.entries(operation.responses ?? {}).forEach(([code, res]) => {
-        const response = resolveIfRef(spec, res);
-        const jsonResponseSchema = response?.content?.['application/json']?.schema;
-        if (jsonResponseSchema && !isRef(jsonResponseSchema) && ["object", "array"].includes(jsonResponseSchema.type!)) {
-          const schemaName = `${_upperFirst(_camelCase(operation.operationId ?? `${path}-${method}`))}${code}Response`;
-          spec.components!.schemas![schemaName] = jsonResponseSchema;
-          response!.content!['application/json'].schema = {
-            $ref: `#/components/schemas/${schemaName}`,
-          };
+    if (operation && typeof operation === "object") {
+      if ("responses" in operation) {
+        Object.entries(operation.responses ?? {}).forEach(([code, res]) => {
+          const response = resolveIfRef(spec, res);
+          const jsonResponseSchema = response?.content?.['application/json']?.schema;
+          if (jsonResponseSchema && !isRef(jsonResponseSchema) && ["object", "array"].includes(jsonResponseSchema.type!)) {
+            const schemaName = `${_upperFirst(_camelCase(operation.operationId ?? `${path}-${method}`))}${code}Response`;
+            spec.components!.schemas![schemaName] = jsonResponseSchema;
+            response!.content!['application/json'].schema = {
+              $ref: `#/components/schemas/${schemaName}`,
+            };
+          }
+        });
+      }
+      if ("requestBody" in operation) {
+        const requestBody = resolveIfRef(spec, operation.requestBody);
+        const jsonRequestSchema = requestBody?.content?.['application/json']?.schema;
+        if (jsonRequestSchema && !isRef(jsonRequestSchema) && ["object", "array"].includes(jsonRequestSchema.type!)) {
+          const schemaName = `${_upperFirst(_camelCase(operation.operationId ?? `${path}-${method}`))}RequestContent`;
+            spec.components!.schemas![schemaName] = jsonRequestSchema;
+            requestBody!.content!['application/json'].schema = {
+              $ref: `#/components/schemas/${schemaName}`,
+            };
         }
-      });
+      }
     }
   }));
 
@@ -749,7 +780,7 @@ const buildData = async (inSpec: OpenAPIV3.Document, metadata: any) => {
 
   // "Inline" any refs to non objects/enums
   const inlinedRefs: Set<string> = new Set();
-  spec = JSON.parse(JSON.stringify(spec, (k, v) => {
+  spec = _cloneDeepWith(spec, (v) => {
     if (v && typeof v === "object" && v.$ref) {
       const resolved = resolveRef(spec, v.$ref);
       if (resolved && resolved.type && resolved.type !== "object" && !(resolved.type === "string" && resolved.enum)) {
@@ -757,8 +788,7 @@ const buildData = async (inSpec: OpenAPIV3.Document, metadata: any) => {
         return resolved;
       }
     }
-    return v;
-  }));
+  });
 
   // Delete the non object schemas that were inlined
   [...inlinedRefs].forEach(ref => {
@@ -782,7 +812,7 @@ const buildData = async (inSpec: OpenAPIV3.Document, metadata: any) => {
   faker.setDefaultRefDate(new Date("2021-06-10"));
   const mockDataContext: MockDataContext = {
     faker,
-    dereferencedSpec: await SwaggerParser.dereference(structuredClone(spec), { dereference: { circular: 'ignore' }}) as OpenAPIV3.Document,
+    dereferencedSpec: await SwaggerParser.dereference(structuredClone(spec), { dereference: { circular: 'ignore' } }) as OpenAPIV3.Document,
   };
 
   // Augment operations with additional data
@@ -945,7 +975,7 @@ const buildData = async (inSpec: OpenAPIV3.Document, metadata: any) => {
         ...model.imports,
         // Include property imports, if any
         ...model.properties.filter(p => p.export === "reference").map(p => p.type),
-      ]));
+      ])).filter(modelImport => modelImport !== model.name); // Filter out self for recursive model references
 
       // Add deprecated flag if present
       (model as any).deprecated = specModel.deprecated || false;
@@ -1006,6 +1036,14 @@ const resolveTemplateDir = (rootScriptDir: string, templateDir: string) => {
   throw new Error(`Template directory ${templateDir} does not exist!`);
 };
 
+export const listFilesInDirRecursive = (dir: string): string[] => {
+  if ((fs.lstatSync(dir)).isDirectory()) {
+    return fs.readdirSync(dir).map((f) => listFilesInDirRecursive(path.join(dir, f))).flatMap(x => x);
+  } else {
+    return [dir];
+  }
+};
+
 export default async (argv: string[], rootScriptDir: string) => {
   const args = parse<Arguments>({
     specPath: { type: String },
@@ -1021,14 +1059,12 @@ export default async (argv: string[], rootScriptDir: string) => {
   const data = await buildData(spec, JSON.parse(args.metadata ?? '{}'));
 
   if (args.printData) {
-    console.log(JSON.stringify(data, null, 2));
+    console.log(util.inspect(data, { depth: 100 }));
   }
 
   // Read all .ejs files in each template directory
-  const templates = args.templateDirs.flatMap(t => fs.readdirSync(resolveTemplateDir(rootScriptDir, t), {
-    recursive: true,
-    withFileTypes: true
-  }).filter(f => f.isFile() && f.name.endsWith('.ejs') && !f.name.endsWith('.partial.ejs')).map(f => path.join(f.parentPath ?? f.path, f.name)));
+  const templates = args.templateDirs.flatMap(t => listFilesInDirRecursive(resolveTemplateDir(rootScriptDir, t))
+    .filter(f => f.endsWith('.ejs') && !f.endsWith('.partial.ejs')));
 
   // Render the templates with the data from the spec
   const renderedFiles = await Promise.all(templates.map(async (template) => {
