@@ -11,7 +11,22 @@ import jsii
 import publication
 import typing_extensions
 
-from typeguard import check_type
+import typeguard
+from importlib.metadata import version as _metadata_package_version
+TYPEGUARD_MAJOR_VERSION = int(_metadata_package_version('typeguard').split('.')[0])
+
+def check_type(argname: str, value: object, expected_type: typing.Any) -> typing.Any:
+    if TYPEGUARD_MAJOR_VERSION <= 2:
+        return typeguard.check_type(argname=argname, value=value, expected_type=expected_type) # type:ignore
+    else:
+        if isinstance(value, jsii._reference_map.InterfaceDynamicProxy): # pyright: ignore [reportAttributeAccessIssue]
+           pass
+        else:
+            if TYPEGUARD_MAJOR_VERSION == 3:
+                typeguard.config.collection_check_strategy = typeguard.CollectionCheckStrategy.ALL_ITEMS # type:ignore
+                typeguard.check_type(value=value, expected_type=expected_type) # type:ignore
+            else:
+                typeguard.check_type(value=value, expected_type=expected_type, collection_check_strategy=typeguard.CollectionCheckStrategy.ALL_ITEMS) # type:ignore
 
 from .._jsii import *
 
@@ -174,7 +189,7 @@ class CdkGraphDiagramPlugin(
         if __debug__:
             type_hints = typing.get_type_hints(_typecheckingstub__a62fd8490a6fe5c8a58c2949b1b834be775be44e10b1e0f512d048d3d48d14d4)
             check_type(argname="argument value", value=value, expected_type=type_hints["value"])
-        jsii.set(self, "bind", value)
+        jsii.set(self, "bind", value) # pyright: ignore[reportArgumentType]
 
     @builtins.property
     @jsii.member(jsii_name="report")
@@ -195,7 +210,7 @@ class CdkGraphDiagramPlugin(
         if __debug__:
             type_hints = typing.get_type_hints(_typecheckingstub__a2478a8d6de8d0b65c7e28e6ee42176f82ac6fa77bb6a501926430454aa9ea04)
             check_type(argname="argument value", value=value, expected_type=type_hints["value"])
-        jsii.set(self, "report", value)
+        jsii.set(self, "report", value) # pyright: ignore[reportArgumentType]
 
 
 @jsii.enum(jsii_type="@aws/pdk.cdk_graph_plugin_diagram.DiagramFormat")
