@@ -11,7 +11,22 @@ import jsii
 import publication
 import typing_extensions
 
-from typeguard import check_type
+import typeguard
+from importlib.metadata import version as _metadata_package_version
+TYPEGUARD_MAJOR_VERSION = int(_metadata_package_version('typeguard').split('.')[0])
+
+def check_type(argname: str, value: object, expected_type: typing.Any) -> typing.Any:
+    if TYPEGUARD_MAJOR_VERSION <= 2:
+        return typeguard.check_type(argname=argname, value=value, expected_type=expected_type) # type:ignore
+    else:
+        if isinstance(value, jsii._reference_map.InterfaceDynamicProxy): # pyright: ignore [reportAttributeAccessIssue]
+           pass
+        else:
+            if TYPEGUARD_MAJOR_VERSION == 3:
+                typeguard.config.collection_check_strategy = typeguard.CollectionCheckStrategy.ALL_ITEMS # type:ignore
+                typeguard.check_type(value=value, expected_type=expected_type) # type:ignore
+            else:
+                typeguard.check_type(value=value, expected_type=expected_type, collection_check_strategy=typeguard.CollectionCheckStrategy.ALL_ITEMS) # type:ignore
 
 from .._jsii import *
 
@@ -108,7 +123,7 @@ class CdkGraphThreatComposerPlugin(
         if __debug__:
             type_hints = typing.get_type_hints(_typecheckingstub__004df50936b7e9d66b1ed9050f1929103969ae05c656b62aa74d1e4e090c0bb1)
             check_type(argname="argument value", value=value, expected_type=type_hints["value"])
-        jsii.set(self, "bind", value)
+        jsii.set(self, "bind", value) # pyright: ignore[reportArgumentType]
 
     @builtins.property
     @jsii.member(jsii_name="report")
@@ -129,7 +144,7 @@ class CdkGraphThreatComposerPlugin(
         if __debug__:
             type_hints = typing.get_type_hints(_typecheckingstub__035069cd2d747ed069143fb7dcd15cc668561fda5e1131e69067e8390c369b5a)
             check_type(argname="argument value", value=value, expected_type=type_hints["value"])
-        jsii.set(self, "report", value)
+        jsii.set(self, "report", value) # pyright: ignore[reportArgumentType]
 
 
 @jsii.data_type(
