@@ -11,7 +11,22 @@ import jsii
 import publication
 import typing_extensions
 
-from typeguard import check_type
+import typeguard
+from importlib.metadata import version as _metadata_package_version
+TYPEGUARD_MAJOR_VERSION = int(_metadata_package_version('typeguard').split('.')[0])
+
+def check_type(argname: str, value: object, expected_type: typing.Any) -> typing.Any:
+    if TYPEGUARD_MAJOR_VERSION <= 2:
+        return typeguard.check_type(argname=argname, value=value, expected_type=expected_type) # type:ignore
+    else:
+        if isinstance(value, jsii._reference_map.InterfaceDynamicProxy): # pyright: ignore [reportAttributeAccessIssue]
+           pass
+        else:
+            if TYPEGUARD_MAJOR_VERSION == 3:
+                typeguard.config.collection_check_strategy = typeguard.CollectionCheckStrategy.ALL_ITEMS # type:ignore
+                typeguard.check_type(value=value, expected_type=expected_type) # type:ignore
+            else:
+                typeguard.check_type(value=value, expected_type=expected_type, collection_check_strategy=typeguard.CollectionCheckStrategy.ALL_ITEMS) # type:ignore
 
 from ..._jsii import *
 
@@ -59,7 +74,7 @@ class _IInputProxy:
         if __debug__:
             type_hints = typing.get_type_hints(_typecheckingstub__1ef17181cbd03403bf29ebca68eff5a8a35f4d949c96cc3fb46f3ea0a800907a)
             check_type(argname="argument value", value=value, expected_type=type_hints["value"])
-        jsii.set(self, "env", value)
+        jsii.set(self, "env", value) # pyright: ignore[reportArgumentType]
 
     @builtins.property
     @jsii.member(jsii_name="fileset")
@@ -71,7 +86,7 @@ class _IInputProxy:
         if __debug__:
             type_hints = typing.get_type_hints(_typecheckingstub__5e40bf4664a9b85473f0cd5ddda585050d4d896dce836a6e5a3bdb3a69d50982)
             check_type(argname="argument value", value=value, expected_type=type_hints["value"])
-        jsii.set(self, "fileset", value)
+        jsii.set(self, "fileset", value) # pyright: ignore[reportArgumentType]
 
     @builtins.property
     @jsii.member(jsii_name="runtime")
@@ -83,7 +98,7 @@ class _IInputProxy:
         if __debug__:
             type_hints = typing.get_type_hints(_typecheckingstub__ebf7ed09213cbf9149b9c6bec9ae709ceed4fe24cc96793660a5facb6d0dfa3e)
             check_type(argname="argument value", value=value, expected_type=type_hints["value"])
-        jsii.set(self, "runtime", value)
+        jsii.set(self, "runtime", value) # pyright: ignore[reportArgumentType]
 
 # Adding a "__jsii_proxy_class__(): typing.Type" function to the interface
 typing.cast(typing.Any, IInput).__jsii_proxy_class__ = lambda : _IInputProxy
@@ -126,7 +141,7 @@ class _INxAffectedConfigProxy:
         if __debug__:
             type_hints = typing.get_type_hints(_typecheckingstub__5de46fc8267401d9897d460c760bbe14b8b04ecd82004f1b6692619e9c2f93aa)
             check_type(argname="argument value", value=value, expected_type=type_hints["value"])
-        jsii.set(self, "defaultBase", value)
+        jsii.set(self, "defaultBase", value) # pyright: ignore[reportArgumentType]
 
 # Adding a "__jsii_proxy_class__(): typing.Type" function to the interface
 typing.cast(typing.Any, INxAffectedConfig).__jsii_proxy_class__ = lambda : _INxAffectedConfigProxy
@@ -224,7 +239,7 @@ class _IProjectTargetProxy:
         if __debug__:
             type_hints = typing.get_type_hints(_typecheckingstub__6bfe01eb9bba696fa0b998df6adfa073377b715413e574acdc20a445a74a157e)
             check_type(argname="argument value", value=value, expected_type=type_hints["value"])
-        jsii.set(self, "dependsOn", value)
+        jsii.set(self, "dependsOn", value) # pyright: ignore[reportArgumentType]
 
     @builtins.property
     @jsii.member(jsii_name="executor")
@@ -237,7 +252,7 @@ class _IProjectTargetProxy:
         if __debug__:
             type_hints = typing.get_type_hints(_typecheckingstub__bbc4539bf826901805777870720a4a4146f122a25e508dfddba2dd794183238b)
             check_type(argname="argument value", value=value, expected_type=type_hints["value"])
-        jsii.set(self, "executor", value)
+        jsii.set(self, "executor", value) # pyright: ignore[reportArgumentType]
 
     @builtins.property
     @jsii.member(jsii_name="inputs")
@@ -258,7 +273,7 @@ class _IProjectTargetProxy:
         if __debug__:
             type_hints = typing.get_type_hints(_typecheckingstub__a4cefac6efec82189278bbac22bc1a2a3940859701ac557425d807f26abacd32)
             check_type(argname="argument value", value=value, expected_type=type_hints["value"])
-        jsii.set(self, "inputs", value)
+        jsii.set(self, "inputs", value) # pyright: ignore[reportArgumentType]
 
     @builtins.property
     @jsii.member(jsii_name="options")
@@ -271,7 +286,7 @@ class _IProjectTargetProxy:
         if __debug__:
             type_hints = typing.get_type_hints(_typecheckingstub__c13f5eef9b35e94985234176670293b661d35e10803d59e5adb76ed143dd2084)
             check_type(argname="argument value", value=value, expected_type=type_hints["value"])
-        jsii.set(self, "options", value)
+        jsii.set(self, "options", value) # pyright: ignore[reportArgumentType]
 
     @builtins.property
     @jsii.member(jsii_name="outputs")
@@ -287,7 +302,7 @@ class _IProjectTargetProxy:
         if __debug__:
             type_hints = typing.get_type_hints(_typecheckingstub__c6827bcd95ea658da76b5b2a114f94af095141e1cb32ee5afd83da4e2eb5611e)
             check_type(argname="argument value", value=value, expected_type=type_hints["value"])
-        jsii.set(self, "outputs", value)
+        jsii.set(self, "outputs", value) # pyright: ignore[reportArgumentType]
 
 # Adding a "__jsii_proxy_class__(): typing.Type" function to the interface
 typing.cast(typing.Any, IProjectTarget).__jsii_proxy_class__ = lambda : _IProjectTargetProxy
@@ -334,7 +349,7 @@ class _ITargetDependencyProxy:
         if __debug__:
             type_hints = typing.get_type_hints(_typecheckingstub__7202bcdf061c1b6995a66da3233b860f29bbae230a28871132e2ed4323c7a435)
             check_type(argname="argument value", value=value, expected_type=type_hints["value"])
-        jsii.set(self, "projects", value)
+        jsii.set(self, "projects", value) # pyright: ignore[reportArgumentType]
 
     @builtins.property
     @jsii.member(jsii_name="target")
@@ -347,7 +362,7 @@ class _ITargetDependencyProxy:
         if __debug__:
             type_hints = typing.get_type_hints(_typecheckingstub__cd5ffe7aa483411b0d1926a1612516a83317c4d8aa90b4b2da85a4747d399586)
             check_type(argname="argument value", value=value, expected_type=type_hints["value"])
-        jsii.set(self, "target", value)
+        jsii.set(self, "target", value) # pyright: ignore[reportArgumentType]
 
 # Adding a "__jsii_proxy_class__(): typing.Type" function to the interface
 typing.cast(typing.Any, ITargetDependency).__jsii_proxy_class__ = lambda : _ITargetDependencyProxy
@@ -391,7 +406,7 @@ class _IWorkspaceLayoutProxy:
         if __debug__:
             type_hints = typing.get_type_hints(_typecheckingstub__0196b26e8b7979766500e7931dc16e4c303ff6d624692565c7de3fde156c74a1)
             check_type(argname="argument value", value=value, expected_type=type_hints["value"])
-        jsii.set(self, "appsDir", value)
+        jsii.set(self, "appsDir", value) # pyright: ignore[reportArgumentType]
 
     @builtins.property
     @jsii.member(jsii_name="libsDir")
@@ -403,7 +418,7 @@ class _IWorkspaceLayoutProxy:
         if __debug__:
             type_hints = typing.get_type_hints(_typecheckingstub__aec58a06dd94ddc5179ffb0ec87eda35d59808e5a5fdf4e884031d7d286b3624)
             check_type(argname="argument value", value=value, expected_type=type_hints["value"])
-        jsii.set(self, "libsDir", value)
+        jsii.set(self, "libsDir", value) # pyright: ignore[reportArgumentType]
 
 # Adding a "__jsii_proxy_class__(): typing.Type" function to the interface
 typing.cast(typing.Any, IWorkspaceLayout).__jsii_proxy_class__ = lambda : _IWorkspaceLayoutProxy
